/* wptCardEdit.h
 *	Copyright (C) 2005 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifndef WPT_CARDEDIT_H
#define WPT_CARDEDIT_H 1

/* All valid card commands. */
enum gpg_editcard_cmd_t {
    GPG_EDITCARD_NAME   = 1,
    GPG_EDITCARD_NAME2  = 2,
    GPG_EDITCARD_KEYURL = 3,
    GPG_EDITCARD_LOGIN  = 4,
    GPG_EDITCARD_SEX    = 5,
    GPG_EDITCARD_LANG   = 6,
    GPG_EDITCARD_APIN   = 7,
    GPG_EDITCARD_UPIN   = 8,
    GPG_EDITCARD_GENKEY = 9,
    GPG_EDITCARD_FETCH  = 10,
    GPG_EDITCARD_CAFPR  = 11,
    GPG_EDITCARD_LIST   = 12,

    GPG_EDITCARD_CHPIN_ID = 20,
    /* change pin */
    GPG_EDITCARD_CHAPIN = 21, /* admin */
    GPG_EDITCARD_CHUPIN = 22, /* user */
    GPG_EDITCARD_UNBPIN = 23 /* unblock PIN */
};

/* Possible card results (can be ORed) */
enum card_result_t {
    GPG_CARDRES_NOCARD = 1, /* no card in the reader */
    GPG_CARDRES_BAD_PIN= 2, /* wrong PIN entered */
    GPG_CARDRES_CANCEL = 4  /* card was missing and user cancelled. */
};

/* gpg edit key abstraction class */
class GpgCardEdit {
private:
    int type;
    int result;
    gpgme_ctx_t ctx;

public:
    GpgCardEdit (void);
    ~GpgCardEdit (void);

public:
    void setPIN (const char *pin);
    void setAdminPIN (const char *admin_pin);
    void setNewPIN (const char *new_pin);
    void setKeygenPassphrase (const char *pass);
    void setCallback (const char* (*cb)(int code, void *opaque), void *cb_value);
    void setResult (int res);
    int  getResult (void);
    void reset (void);
    int getType ();

    gpgme_error_t getCardStatus (gpg_card_t *r_card);
    gpgme_error_t genKey (int flags, const char *name,
			  const char *email, const char *comment,
			  long valid, char **r_key_fpr);
    gpgme_error_t changePIN (int type);

    gpgme_error_t updateName (const char *given, const char *sur);
    gpgme_error_t updateURL (const char *url);
    gpgme_error_t updateLogin (const char *login);
    gpgme_error_t updateLanguage (const char *lang);
    gpgme_error_t updateSex (char sex);

    gpgme_error_t doCmd (int cmd, const char *arg1, const char *arg2);

    gpgme_error_t fetchKey (void);

public: /* XXX hide the internals */    
    const char *pin;
    const char *pin_new;
    const char *admin_pin;

    int cnt;
    int cancel;

    struct {
	const char *surname;
	const char *givenname;
	const char *keyurl;
	const char *login;
	const char *lang;	
	char        sex;
    } edit;

    struct {
	char *name;
	char *email;
	char *comment;
	char *expdate;
	int  flags;
	const char *pass;
	char *key_fpr;
    } keygen;

    const char *(*card_cb) (int code, void * opaque);
    void *cb_value;
};


gpg_card_t    gpg_card_load (void);
void          gpg_card_release (gpg_card_t card);
gpgme_error_t gpg_card_edit (gpgme_ctx_t ctx, GpgCardEdit *ce);

#endif
